/*----------------------------------------------------------------------------
	This is a Virtual Desktop Manager for use under Win32 Shells that
	support the LiteStep 0.24.5 module standard.

	Copyright (C) 2001-2003 Chris Rempel

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
----------------------------------------------------------------------------*/

/*---System Includes--------------------------------------------------------*/
#include <tchar.h>
#include <windows.h>

/*---Module Includes--------------------------------------------------------*/
#include "common.h"

/*--------------------------------------------------------------------------*/

STRINGIDMAP g_stretchMap[] = {
	 { _T(".vertical"),   eBVDM_STRETCH__VERTICAL }
	,{ _T(".horizontal"), eBVDM_STRETCH__HORIZONTAL }
	,{ _T(".both"),       eBVDM_STRETCH__BOTH }
	,{ NULL, 0 }
};

STRINGIDMAP g_alignMap[] = {
	 { _T(".left"),   eBVDM_ALIGN__LEFT }
	,{ _T(".right"),  eBVDM_ALIGN__RIGHT }
	,{ _T(".top"),    eBVDM_ALIGN__TOP }
	,{ _T(".bottom"), eBVDM_ALIGN__BOTTOM }
	,{ _T(".center"), eBVDM_ALIGN__CENTER }
	,{ _T(".middle"), eBVDM_ALIGN__MIDDLE }
	,{ NULL, 0 }
};

/*--------------------------------------------------------------------------*/
/* From utility.c */
EXTERN_C VOID focusLast( VOID );
EXTERN_C VOID switchToVD( UINT const );
EXTERN_C VOID gatherToVD( UINT const, char const );
EXTERN_C VOID stretchWnd( HWND const, enum EbvdmStretchMode const );
EXTERN_C VOID alignWnd( HWND const, enum EbvdmAlignMode const );
EXTERN_C VOID setNumDesks( UINT const );
EXTERN_C VOID moveActiveWndTo( UINT const );

EXTERN_C UINT g_unCurDesk;
EXTERN_C UINT g_unDesks;


/*--------------------------------------------------------------------------*/
INT getVD( LPCTSTR const args )
{
	INT retVal = -1;

	if(NULL != args)
	{
		if(_istdigit(*args))
		{
			retVal = _ttoi(args);
		}
		else if(_T('.') == *args)
		{
			if(0 == _tcsicmp(_T(".home"), args))
			{
				retVal = VD_FIRST;
			}
			else if(0 == _tcsicmp(_T(".end"), args))
			{
				retVal = VD_LAST;
			}
			else if(0 == _tcsicmp(_T(".next"), args))
			{
				retVal = VD_NEXT(g_unCurDesk);
			}
			else if (0 == _tcsicmp(_T(".prev"), args))
			{
				retVal = VD_PREV(g_unCurDesk);
			}
		}
	}

	if(!VD_ISVALID(retVal))
	{
		retVal = g_unCurDesk;
	}

	return retVal;
}

/*----------------------------------------------------------------------------
 Bang Commands
----------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
VOID bVDMSwitchTo( HWND caller, LPCTSTR args )
{
	switchToVD(getVD(args));

	return;
	UNREFERENCED_PARAMETER( caller );
}

/*--------------------------------------------------------------------------*/
VOID bVDMNext( HWND caller, LPCTSTR args )
{
	switchToVD(VD_NEXT(g_unCurDesk));

	return;
	UNREFERENCED_PARAMETER( caller );
	UNREFERENCED_PARAMETER( args );
}

/*--------------------------------------------------------------------------*/
VOID bVDMPrev( HWND caller, LPCTSTR args )
{
	switchToVD(VD_PREV(g_unCurDesk));

	return;
	UNREFERENCED_PARAMETER( caller );
	UNREFERENCED_PARAMETER( args );
}

/*--------------------------------------------------------------------------*/
VOID bVDMGather( HWND caller, LPCTSTR args )
{
	gatherToVD(getVD(args), VD_HORZVERT);

	return;
	UNREFERENCED_PARAMETER( caller );
}

/*--------------------------------------------------------------------------*/
VOID bVDMGatherH( HWND caller, LPCTSTR args )
{
	gatherToVD(getVD(args), VD_HORIZONTAL);

	return;
	UNREFERENCED_PARAMETER( caller );
}

/*--------------------------------------------------------------------------*/
VOID bVDMGatherV( HWND caller, LPCTSTR args )
{
	gatherToVD(getVD(args), VD_VERTICAL);

	return;
	UNREFERENCED_PARAMETER( caller );
}

/*--------------------------------------------------------------------------*/
VOID bVWMDesks( HWND caller, LPCTSTR args )
{

	if(NULL != args && isdigit(*args))
	{
		const UINT tmp = _ttoi(args);

		if(tmp != g_unDesks)
		{
			setNumDesks(tmp);
		}
	}

	return;
	UNREFERENCED_PARAMETER( caller );
}

/*--------------------------------------------------------------------------*/
VOID bVDMMoveWindow( HWND caller, LPCTSTR args )
{
	moveActiveWndTo(getVD(args));

	return;
	UNREFERENCED_PARAMETER( caller );
}

/*--------------------------------------------------------------------------*/
VOID bVDMWindowStretch( HWND caller, LPCTSTR args )
{
	if(NULL != args && _T('.') == *args)
	{
		HWND hWnd1;
		HWND hWnd2;

		hWnd1 = GetForegroundWindow();
		hWnd2 = GetLastActivePopup(hWnd1);

		if(hWnd1 == hWnd2)
		{
			PSTRINGIDMAP pMap;

			for(pMap = g_stretchMap; NULL != pMap->pszStr; pMap++)
			{
				if(0 == _tcsicmp(pMap->pszStr, args))
				{
					stretchWnd(hWnd1, pMap->unID);
					break;
				}
			}
		}
	}

	return;
	UNREFERENCED_PARAMETER( caller );
}

/*--------------------------------------------------------------------------*/
VOID bVDMWindowAlign( HWND caller, LPCTSTR args )
{
	if(NULL != args && _T('.') == *args)
	{
		HWND hWnd1;
		HWND hWnd2;

		hWnd1 = GetForegroundWindow();
		hWnd2 = GetLastActivePopup(hWnd1);

		if(hWnd1 == hWnd2)
		{
			PSTRINGIDMAP pMap;

			for(pMap = g_alignMap; NULL != pMap->pszStr; pMap++)
			{
				if(0 == _tcsicmp(pMap->pszStr, args))
				{
					alignWnd(hWnd1, pMap->unID);
					break;
				}
			}
		}
	}

	return;
	UNREFERENCED_PARAMETER( caller );
}

/*--------------------------------------------------------------------------*/
VOID bVDMWindowMaximize( HWND caller, LPCTSTR args )
{
	HWND hWnd1;
	HWND hWnd2;

	hWnd1 = GetForegroundWindow();
	hWnd2 = GetLastActivePopup(hWnd1);

	if(hWnd1 == hWnd2)
	{
		ShowWindow(hWnd1, SW_MAXIMIZE);
	}

	return;
	UNREFERENCED_PARAMETER( caller );
	UNREFERENCED_PARAMETER( args );
}

/*--------------------------------------------------------------------------*/
VOID bVDMWindowMinimize( HWND caller, LPCTSTR args )
{
	HWND hWnd1;
	HWND hWnd2;

	hWnd1 = GetForegroundWindow();
	hWnd2 = GetLastActivePopup(hWnd1);

	if(hWnd1 == hWnd2)
	{
		ShowWindow(hWnd1, SW_SHOWMINNOACTIVE);
	}

	focusLast();

	return;
	UNREFERENCED_PARAMETER( caller );
	UNREFERENCED_PARAMETER( args );
}

/*--------------------------------------------------------------------------*/
VOID bVDMWindowRestore( HWND caller, LPCTSTR args )
{
	HWND hWnd1;
	HWND hWnd2;

	hWnd1 = GetForegroundWindow();
	hWnd2 = GetLastActivePopup(hWnd1);

	if(hWnd1 == hWnd2)
	{
		ShowWindow(hWnd1, SW_SHOWNOACTIVATE);
	}

	return;
	UNREFERENCED_PARAMETER( caller );
	UNREFERENCED_PARAMETER( args );
}

/*--------------------------------------------------------------------------*/
VOID registerBangCmds( VOID )
{
	AddBangCommand(_T("!BVWMSWITCHTO"), bVDMSwitchTo);
	AddBangCommand(_T("!BVWMNEXT"), bVDMNext);
	AddBangCommand(_T("!BVWMPREV"), bVDMPrev);
	AddBangCommand(_T("!BVWMGATHER"), bVDMGather);
	AddBangCommand(_T("!BVWMGATHERH"), bVDMGatherH);
	AddBangCommand(_T("!BVWMGATHERV"), bVDMGatherV);
	AddBangCommand(_T("!BVWMDESKS"), bVWMDesks);
	AddBangCommand(_T("!BVWMMOVEWINDOW"), bVDMMoveWindow);
	AddBangCommand(_T("!BVWMWINDOWSTRETCH"), bVDMWindowStretch);
	AddBangCommand(_T("!BVWMWINDOWALIGN"), bVDMWindowAlign);
	AddBangCommand(_T("!BVWMWINDOWMAXIMIZE"), bVDMWindowMaximize);
	AddBangCommand(_T("!BVWMWINDOWMINIMIZE"), bVDMWindowMinimize);
	AddBangCommand(_T("!BVWMWINDOWRESTORE"), bVDMWindowRestore);

	return;
}

/*--------------------------------------------------------------------------*/
VOID unregisterBangCmds( VOID )
{
	RemoveBangCommand(_T("!BVWMSWITCHTO"));
	RemoveBangCommand(_T("!BVWMNEXT"));
	RemoveBangCommand(_T("!BVWMPREV"));
	RemoveBangCommand(_T("!BVWMGATHER"));
	RemoveBangCommand(_T("!BVWMGATHERH"));
	RemoveBangCommand(_T("!BVWMGATHERV"));
	RemoveBangCommand(_T("!BVWMDESKS"));
	RemoveBangCommand(_T("!BVWMMOVEWINDOW"));
	RemoveBangCommand(_T("!BVWMWINDOWSTRETCH"));
	RemoveBangCommand(_T("!BVWMWINDOWALIGN"));
	RemoveBangCommand(_T("!BVWMWINDOWMAXIMIZE"));
	RemoveBangCommand(_T("!BVWMWINDOWMINIMIZE"));
	RemoveBangCommand(_T("!BVWMWINDOWRESTORE"));

	return;
}

/*--------------------------------------------------------------------------*/
